/*
 * rs_relay_soft.c
 *
 * Created: 2012-10-24 14:40:52
 *  Author: dsosnowski
 */ 


//includes
#include <avr/io.h>
#include <avr/interrupt.h>
//#include <avr/sleep.h>
//#include <avr/pgmspace.h>
#include "s_uart.h"
#include "ee_mem.h"
#include "wdog.h"

#define F_CPU 4000000UL
#include <util/delay.h>

//extrnal signals
//PORTA
#define LED 7 //H - on, L - off
#define MOSI 6
#define MISO 5
#define SCK 4
#define TXD 3 //output
#define TEN 2 //H - tx enable, L - rx enable
#define RXD 1 //input
#define REL1 0 //REL #1

//PORTB
#define RTS 3 //external RESET
#define REL2 2 //REL #2
#define X2 1 //crystal resonator
#define X1 0 //crystal resonator

#define LED_ON 1
#define LED_OFF 0
#define BAUD_FACTOR 208

#define LED_PERIOD	15

//subs
void HartInit(void);
void adr2str(void);
void StatePrepare(char adr);
void AddressPrepare(char adr);
void StateSet(char adr, char st);
char str2adr(char *pt);


//var
/*char frame[25] = {'T', '0', '1', '=', ' ', '2', '5', '"', 'C', '\r',
				  'T', '0', '2', '=', ' ', '3', '1', '"', 'C', '\r'};
/R01=1
R01?
*/

extern struct {
	volatile char s_byte;
	volatile char s_index;
	volatile char r_buffer[SUART_BUFFER];
	volatile char r_pointer;
	volatile char r_index;
	volatile char u_status;
} suart;



char frame[25];
char address;
char need_address;
char process;
char state;

char actual[] = {'\r', 'A', 'c', 't', 'u', 'a', 'l', ' ', 'A', 'd', 'r', ':', 0};
char enter[] = {'\r', 'E', 'n', 't', 'e', 'r', ' ', 'n', 'e', 'w',' ', 'A', 'D', 'R', 0};



int main(void)
{
	 char *read_pt;
	 char i = 0;

     HartInit();
	 _delay_ms(200);
	 SuartInit();

	 //SuartSend("Hello! ");

	//----------------------------------------------------- 
	//restore address
	 address = ee_read(15); 

	//get default address
	if ((PINA & (1<<MOSI)) == 0)
	{
		ee_write(15, 0); 
		i = 1;
	}	
	
	//----------------------------------------------------- 	
	//enter new address
	if ((PINA & (1<<SCK)) == 0)
	{
		 adr2str();
		 SuartSend(&actual[0]);
		 SuartSend(&frame[0]);
		 SuartSend(&enter[0]);
		 SuartClear();
		 read_pt = SuartRead();
		 read_pt++;
		 read_pt++;
		 *read_pt = 0;
		 while (*read_pt == 0)
			_delay_ms(200);
		 read_pt--;
		 address = (*read_pt - '0');
		 read_pt--;
		 address += (10 * (*read_pt - '0'));
	 	 if (address > 98) address = 0;
		 ee_write(15, address); 
		 i = 1;
	}
	
	//----------------------------------------------------- 
	 WdtSet(7);
	 SuartClear();
	 process = 0;
	 address = ee_read(15); 
	 if (address > 98) address = 0;

	//----------------------------------------------------- 
	//send address  
	 if (i)
	 {		 
		 adr2str(); 
		 SuartSend(&actual[0]);
		 SuartSend(&frame[0]);
		 _delay_ms(200);
	 }

 	 SuartClear();
	 SuartSend(&suart.r_buffer[0]);	  
     while(1)
     {	
	//--------------------------------------------------
		 read_pt = SuartRead();
		 if (*read_pt == 'R')
		 {
			//check address
			read_pt++;
			if (str2adr(read_pt))		
			{
				//need adr = get address
				read_pt++;
				read_pt++;
				//---?--------------------------
				/*if (*read_pt == '?')
				{ 
					read_pt++;
					process = 1;		//read state	
				}*/
				//---=--------------------------								
				if (*read_pt == '=')
				{
					read_pt++;
					state = *read_pt;					
					read_pt++;
					//process = 2;		//set state
					if (state == '?') process = 1;
					if ((state == '0') || (state == '1')) process = 2;
				} 		

				if (*read_pt == '\r') *read_pt = 0; //if enter then clear - no double command
				else process = 0;					//not enter - clear command
			}
			if (process) SuartClear();
		 }		
		 
	//--------------------------------------------------
		 if (process)
		 {
			if ((need_address == address) || (need_address == (address + 1)))
			{
				PORTA |= (1<<LED);
			
				if (process == 1)		//read
				{
					AddressPrepare(need_address);
					StatePrepare(need_address);
					SuartSend(&frame[0]);
				}
				
				if (process == 2)		//set
				{
					StateSet(need_address, state);
				}					
				_delay_ms(300);
			}
			process = 0;
		}		 
		 
	//--------------------------------------------------
		 i++;
		 if (i > LED_PERIOD)
		 {
				i = 0;
				if ((PINA & (1<<MISO)) == 0)
				{
					process = 1;
					need_address = address;					
				} 		 
				PORTA |= (1<<LED);
		 }
		 _delay_ms(200);
		 WdtReset();
		 PORTA &= ~(1<<LED); 		
	}	 	
}

//------------------------------------------
void HartInit(void)
{
     PORTA = (1<<LED | 1<<MOSI | 1<<MISO | 1<<SCK | 1<<TXD | 1<<RXD);
     DDRA = (1<<LED | 1<<TXD | 1<<TEN | 1<<REL1);
     PORTB = 0;
     DDRB = (1<<REL2);
	 PRR = (1<<PRADC | 1<<PRUSI | 1<<PRTIM1);
	 ACSR = (1<<ADC);
}

//------------------------------------------
void adr2str(void)
{
	 frame[0] = ((address / 10) + '0');
	 frame[1] = ((address % 10) + '0');
	 frame[2] = 0;
}

//------------------------------------------
char str2adr(char *pt)
{
	 char temp;
	 if (*pt >= '0' || *pt <= '9') 
	 {
		 temp = (*pt - '0');
		 need_address = (temp * 10);
		 pt++;
		 if (*pt >= '0' || *pt <= '9') 
		 {
			temp = (*pt - '0');
			need_address += temp;
			return 1;
		 }
	 }
	 need_address = 0;	 
	 return 0;	 		 
}

//------------------------------------------
void AddressPrepare(char adr)
{
	char temp;
	char *pt;
	
	pt = &frame[0];
	*pt = ('R'); pt++;
	temp = (adr / 10);
	if (temp > 9) temp = 9;
	*pt = ('0' + temp); pt++;
	temp = (adr % 10);
	*pt = ('0' + temp); pt++;
	*pt = ('=');
}

//------------------------------------------
void StatePrepare(char adr)
{
	frame[4] = 0;
	if (adr == address) 
	{
		if (PINA & (1<<REL1)) 
			frame[4] = '1';
		else 
			frame[4] = '0';
	}
	if (adr == (address + 1)) 
	{
		if (PINB & (1<<REL2)) 
			frame[4] = '1';
		else 
			frame[4] = '0';
	}
	frame[5] = ('\r');
	frame[6] = 0;
}			

//------------------------------------------
void StateSet(char adr, char st)
{
	if (adr == address) 
	{
		if (st == '1') PORTA |= (1<<REL1);
		if (st == '0') PORTA &= ~(1<<REL1);
		if (st == 'C')
		{
			PORTA &= ~(1<<REL1);
			PORTB &= ~(1<<REL2);					
		} 
		if (st == 'S')
		{
			PORTA |= (1<<REL1);
			PORTB |= (1<<REL2);					
		} 		
	}
	if (adr == (address + 1)) 
	{
		if (st == '1') PORTB |= (1<<REL2);
		if (st == '0') PORTB &= ~(1<<REL2);
	}					
}			

