




#include <avr/io.h>
#include <avr/interrupt.h>
#define F_CPU 4000000UL
#include <util/delay.h>
#include "s_uart.h"



#define CORE_CLOCK		4000000
#define BAUD_RATE		9600

#define TIMER_DIV		8
#define TIMER_CLOCK		(CORE_CLOCK/TIMER_DIV)
#define BAUD_FACTOR		(TIMER_CLOCK/BAUD_RATE)	//1 int on 1 bit

#define SUART_PORT		PORTA
#define SUART_PIN		PINA
#define SUART_DDR		DDRA
#define TXD 			3 //output
#define TEN 			2 //H - tx enable, L - rx enable
#define RXD 			1 //input

#define TXD_SET			SUART_PORT |= (1<<TXD)
#define TXD_CLR			SUART_PORT &= ~(1<<TXD)
#define TXD_EN			SUART_PORT |= (1<<TEN)
#define TXD_DIS			SUART_PORT &= ~(1<<TEN)

#define START_TIMER		TCCR0B = (1<<CS01 | 1<<WGM02)
#define STOP_TIMER		TCCR0B = 0
#define TIMER_VAL		TCNT0
#define CLEAR_TIMER		TCNT0 = 0

#define DISABLE_RXD		GIMSK = 0
#define CLEAR_RXD	 	GIFR = (1<<PCIF0)
#define ENABLE_RXD		GIMSK = (1<<PCIE0)


//#define SUART_BUFFER	32

#define LED 7 //H - on, L - off


//i.e        |         |          |
//"VV" = -----__--_-_-_-__--_-_-_------



//-------------------------------------------
struct {
	volatile char s_byte;
	volatile char s_index;
	volatile char r_buffer[SUART_BUFFER];
	volatile char r_pointer;
	volatile char r_index;
	volatile char u_status;
} suart;




//-------------------------------------------
void SuartInit(void)
{
	 SUART_PORT |= (1<<TXD | 1<<RXD);		//config ports
     SUART_DDR |= (1<<TXD | 1<<TEN);
   
     TCCR0A = (1<<WGM00 | 1<<WGM01); 		//fast PWM - TOP = OCR0A
     TCCR0B = (1<<WGM02); 					//fast PWM
	 OCR0A = BAUD_FACTOR;					//S_uart freq  

	 TIFR0 = (1<<TOV0);					//clear flag
 	 TIMSK0 = (1<<TOIE0);					//enable int		

	 PCMSK0 = (1<<PCINT1);					//enable pin change int on pina.1
	 GIFR = (1<<PCIF0);						//clear flag
	 GIMSK = (1<<PCIE0);					//enable pin change int
	
	 //START_TIMER;
	 
	 sei();
}

//-------------------------------------------
void SuartSend(char *data)
{
	DISABLE_RXD;
	if (*data) TXD_EN;	

	//_delay_ms(2);

	CLEAR_TIMER;					//first - idle cycle
	START_TIMER;					//start

	while(*data)
	{
 		suart.u_status = 1;			//transmit mode
		suart.s_byte = 0;			//start bit first
		suart.s_index = 10;			//set cycles
		while(suart.s_index >= 10);	//wait for star bit
		suart.s_byte = *data;		//data
		while(suart.s_index > 1);		//wait for complete and next
		suart.s_byte = 0xFF;			//stop_bit
		while(suart.s_index > 0);		//wait for complete and next
		data++;						//next data
	}
	suart.u_status = 0;				//receive mode
	STOP_TIMER;
	//_delay_ms(2);

	TXD_DIS;
	CLEAR_RXD;
	ENABLE_RXD;	
}

//-------------------------------------------
char* SuartRead(void)
{
	return &suart.r_buffer[0];
}

//-------------------------------------------
void SuartClear(void)
{
	suart.r_buffer[0] = 0;
	suart.r_buffer[1] = 0;
	suart.r_buffer[2] = 0;
	suart.r_pointer = 0;
	suart.u_status = 0;				//receive mode
}

//-------------------------------------------
SIGNAL(TIM0_OVF_vect)
{	      
//	static char r_index;
	static char r_byte = 0xFF;
	
	//---------------------------------------------
	if (suart.u_status == 1)				//if any byte to send
	{
		if (suart.s_byte & 1) TXD_SET; 
		else		   		  TXD_CLR;
		suart.s_byte>>=1;			//next bit
		suart.s_index--;			//decrement counter
	}
	
	//---------------------------------------------
	if (suart.u_status == 2)				//if any byte to send
	{
		//---------------------------------------------
		if (suart.r_index < 10)
		{		
			suart.r_index++;										//increment counter
			if (SUART_PIN & (1<<RXD)) r_byte += 0x80;		//set bit 7
			if (suart.r_index == 9)
			{
				if (r_byte == 'R') suart.r_pointer = 0;
				suart.r_buffer[suart.r_pointer] = r_byte;
				if (suart.r_pointer < (SUART_BUFFER - 1)) suart.r_pointer++;
				//else suart.r_pointer = 0;
				suart.r_buffer[suart.r_pointer] = 0;		
			}
			r_byte >>= 1;									//at the end - shift
		} 												//control counter - max bits
		//---------------------------------------------
		if (suart.r_index >= 10)
		{
			suart.u_status = 0;
			STOP_TIMER;
			CLEAR_RXD;
			ENABLE_RXD;	
		}
	}
}


//-------------------------------------------
SIGNAL(PCINT0_vect)
{	 
	if ((SUART_PIN & (1<<RXD)) == 0)     
	{
		DISABLE_RXD;
		suart.u_status = 2;
		suart.r_index = 0;
		TIMER_VAL = (BAUD_FACTOR / 2);
		START_TIMER;
	}	
}
